from django.conf import settings

from .choices import (CURRENCY_CHOICES as BASE_CURRENCY_CHOICES,
                      CURRENCY_LABELS as BASE_CURRENCY_LABELS,
                      CURRENCY_BY_COUNTRIES as BASE_CURRENCY_BY_COUNTRIES)


CURRENCY_COOKIE_NAME = getattr(settings, 'METASETTINGS_CURRENCY_COOKIE_NAME', 'django_currency')

DEFAULT_CURRENCY = getattr(settings, 'METASETTINGS_DEFAULT_CURRENCY', 'EUR')

CURRENCY_CHOICES = getattr(settings, 'METASETTINGS_CURRENCY_CHOICES', BASE_CURRENCY_CHOICES)

CURRENCY_LABELS = getattr(settings, 'METASETTINGS_CURRENCY_LABELS', BASE_CURRENCY_LABELS)

CURRENCY_BY_COUNTRIES = getattr(settings, 'METASETTINGS_CURRENCY_BY_COUNTRIES', BASE_CURRENCY_BY_COUNTRIES)
